/*
 * CzekanowskiS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-14
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * CZEKANOWSKI SIMILARITY
 * 
 * <br>http://ag.arizona.edu/classes/rnr555/lecnotes/10.html
 * 
 * @author  Martin Hassel
 * @version 2004-Sep-14
 */
public class CzekanowskiS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		double x, y, sum1, sum2, sum3;
		double sim_matrix[][]; /* matrix to contain the distance */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate distance matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 200 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 200.0;
		
		/* Begin calculations */
		for(int row1 = 0; row1 < rows; ++row1) {
			for(int row2 = (row1 + 1); row2 < rows; ++row2) {
				sum1 = sum2 = sum3 = 0.0;
				for(int column = 0; column < columns; ++column) {
					x = data_matrix[row1][column];
					y = data_matrix[row2][column];
					sum1 += x < y ? x : y;
					sum2 += x;
					sum3 += y;
				}
				sim_matrix[row1][row2] = sim_matrix[row2][row1] = 
					(200.0*sum1) / (double)(sum2+sum3);
			}
		}
		return sim_matrix;		
	}
}
