/*
 * DiceS_row.java
 * Copyright (c) 2004, KTH NADA.
 *
 * This file is part of SweSum^2 (see http://swesum.nada.kth.se/),
 * and is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, see
 * http://opensource.org/licenses/gpl-license.php
 *
 * Martin Hassel, 2004-Sep-03
 * http://www.nada.kth.se/~xmartin/
 *
 */

package moj.similarity.measures;

import moj.similarity.SimilarityMeasure;

/**
 * DICE SIMILARITY COEFFICIENT
 * 
 * @author  Martin Hassel
 * @version 2004-Sep-03
 */
public class DiceS_row extends SimilarityMeasure {
	/**
	 * @param data_matrix an array of <code>float</code> arrays denoting the 
	 *        rows in the matrix that are to be compared for similarity.
	 * @return an array of <code>double</code> arrays containing the 
	 *         similarities between all rows
	 */
	public double[][] analyzeMatrix(float[][] data_matrix) {
		float posmatch = 0.0f; /* positive matches (taxa in pa matix lists) */
		float mismatch = 0.0f; /* non matches  (taxa in pa matix lists) */
		double[][] sim_matrix; /* matrix to contain the similarity */
		int rows = data_matrix.length;
		int columns = data_matrix[0].length;
		
		/* allocate similarity matrix */
		sim_matrix = new double[rows][rows];
		
		/* Set diagonal to 1 */
		for(int i = 0; i < rows; ++i)
			sim_matrix[i][i] = 1.0;
		
		/* Begin calculations */
		for(int row = 0; row < rows; ++row) {
			for(int column = (row + 1); column < columns; ++column) {
				posmatch = 0;
				mismatch = 0;
				
				for(int taxa = 0; taxa < columns; ++taxa) {
					if(data_matrix[row][taxa] == 1 && data_matrix[column][taxa] == 1)
						posmatch = posmatch + 1;
					else if(data_matrix[row][taxa] != data_matrix[column][taxa])
						mismatch = mismatch + 1;
				}
				sim_matrix[column][row] = sim_matrix[row][column] = 
					(2.0*posmatch) / (2.0*posmatch+mismatch);
			}
		}
		return sim_matrix;
	}
}
